import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.streaming.Trigger;
import org.apache.spark.sql.types.StructType;
import org.bson.Document;

public class SparkCSVConsumer {

    public static void main(String[] args) throws TimeoutException, StreamingQueryException {
        //Create and get Spark session
        SparkSession spark = getSparkSession();

        //Read data from Kafka
        Dataset<Row> rawJsonDataFrame = loadDatafromKafka(spark);

        Dataset<Row> df = convertingJsonDatasetToSchemaDataset(rawJsonDataFrame);

        //Filtering dataset based on visa cardType
        Dataset<Row> filteredDataFrame = df.filter(functions.col("deliveryStatus").equalTo("Shipped"));

        //Converting DataFrame to Dataset
        Dataset<ProductInfo> productInfoDataset = filteredDataFrame.as(Encoders.bean(ProductInfo.class));

        //Writing data to mongo db
        writeDataToMongoDB(productInfoDataset);
    }

    private static Dataset<Row> loadDatafromKafka(SparkSession spark) {
        Dataset<Row> rawDataFrame = spark
                .readStream()
                .format("kafka")
                .options(constructKafkaConfigMap())
                .load();
        return rawDataFrame;
    }

    private static Map<String, String> constructKafkaConfigMap() {
        Map<String, String> kafkaConfigMap = new HashMap<>();
        kafkaConfigMap.put("kafka.bootstrap.servers", "localhost:9092");
        kafkaConfigMap.put("subscribe", "ProductInfoTopic");
        kafkaConfigMap.put("startingOffsets", "earliest");
        kafkaConfigMap.put("failOnDataLoss", "true");
        kafkaConfigMap.put("kafka.con.group.id", "local");
        return kafkaConfigMap;
    }

    private static SparkSession getSparkSession() {
        SparkSession spark = SparkSession.builder()
                .master("local")
                .appName("SparkConsumer")
                .config("spark.mongodb.output.uri", "mongodb://127.0.0.1/userdb.users_info")
                .getOrCreate();
        return spark;
    }

    private static Dataset<Row> convertingJsonDatasetToSchemaDataset(Dataset<Row> rawItemDF) {
        StructType structType = Encoders.bean(ProductInfo.class).schema();

        Dataset<Row> df = rawItemDF
                .selectExpr("CAST(value as String) as message")
                .select(functions.from_json(functions.col("message"), structType).as("data"))
                .select("data.*");
        return df;
    }

    private static void writeDataToMongoDB(Dataset<ProductInfo> ds)
            throws TimeoutException, StreamingQueryException {
        StreamingQuery query = ds.writeStream().outputMode(OutputMode.Append())
                .option("checkpointLocation", "checkpointlocaion/streamingjob")
                .trigger(Trigger.Continuous("10 second"))
                .foreach(new ForeachWriter<ProductInfo>() {
                    private MongoClient mongoClient;
                    private MongoDatabase database;
                    private MongoCollection<Document> collection;

                    @Override
                    public boolean open(long partitionId, long epochId) {
                        mongoClient = MongoClients.create("mongodb://localhost:27017");
                        database = mongoClient.getDatabase("userdb");
                        collection = database.getCollection("productinfo");
                        return true;
                    }

                    @Override
                    public void process(ProductInfo productInfo) {
                        Document document = new Document();
                        document.append("productId", productInfo.getProductId());
                        document.append("productName", productInfo.getProductName());
                        document.append("productPrice", productInfo.getProductPrice());
                        document.append("deliveryStatus", productInfo.getDeliveryStatus());
                        // Insert the document into the collection
                        collection.insertOne(document);

                    }

                    @Override
                    public void close(Throwable errorOrNull) {
                        // Close the MongoDB connection
                        mongoClient.close();
                    }
                }).start();
        query.awaitTermination();
    }
}
